#include "tdb2-source.h"
#include <ccan/tap/tap.h>
#include <stdlib.h>
#include <err.h>
#include "logging.h"

int main(int argc, char *argv[])
{
	struct tdb_context *tdb;
	TDB_DATA key, data;

	plan_tests(13);
	tdb = tdb1_open("run-check.tdb", 1, TDB_DEFAULT,
			O_CREAT|O_TRUNC|O_RDWR, 0600, &tap_log_attr);

	ok1(tdb);
	ok1(tdb1_check(tdb, NULL, NULL) == 0);

	key.dsize = strlen("hi");
	key.dptr = (void *)"hi";
	data.dsize = strlen("world");
	data.dptr = (void *)"world";

	ok1(tdb1_store(tdb, key, data, TDB_INSERT) == 0);
	ok1(tdb1_check(tdb, NULL, NULL) == 0);
	tdb1_close(tdb);

	tdb = tdb1_open("run-check.tdb", 1024, 0, O_RDWR, 0,
			&tap_log_attr);
	ok1(tdb);
	ok1(tdb1_check(tdb, NULL, NULL) == 0);
	tdb1_close(tdb);

	tdb = tdb1_open("test/tdb1.corrupt", 1024, 0, O_RDWR, 0,
			&tap_log_attr);
	ok1(tdb);
	ok1(tdb1_check(tdb, NULL, NULL) == -1);
	ok1(tdb_error(tdb) == TDB_ERR_CORRUPT);
	tdb1_close(tdb);

	/* Big and little endian should work! */
	tdb = tdb1_open("test/old-nohash-le.tdb1", 1024, 0, O_RDWR, 0,
			&tap_log_attr);
	ok1(tdb);
	ok1(tdb1_check(tdb, NULL, NULL) == 0);
	tdb1_close(tdb);

	tdb = tdb1_open("test/old-nohash-be.tdb1", 1024, 0, O_RDWR, 0,
			&tap_log_attr);
	ok1(tdb);
	ok1(tdb1_check(tdb, NULL, NULL) == 0);
	tdb1_close(tdb);

	return exit_status();
}
